/******************************************************************************
*
* TI Clock Hook, programmed by Johannes Zeppenfeld
* adapted from the tigcc clock tutorial by Thomas Nussbaumer
*
* WARNING!!! Do NOT run this code directly on your calc. It will probably crash!
*							Use the installer instead.
*
*******************************************************************************
* For the original TICT tutorial this code was based off, visit:
* http://tigcc.ticalc.org/tut/
* ...tsrclock.html
* ...tsrclock_source.txt
*******************************************************************************
* You may use this code however you like, but please give credit to those who
* wrote it.
* Other than that, this code is e-note-ware: if you use it, please send me an
* email telling me about your project and how you integrated the clock into it.
* I love to hear what others use programs I write for.
* Johannes Zeppenfeld (jzeppenfeld@excite.com)
******************************************************************************/
//#define TEST_W_5  //uncomment to test clock with AUTO_INT_5 (on VTI)

#include <tigcclib.h>

#ifndef EnableAutoInt3  //for people using less recent versions of tigcc, these values are not defined:
#define pokeIO_bclr(__dest_addr__,__bit_nr__) ({register char *__addr_temp__=(char *)(__dest_addr__);asm("bclr.b %2,%0":"+dm"(*__addr_temp__):"0"(*__addr_temp__),"di"(__bit_nr__));})
#define pokeIO_bset(__dest_addr__,__bit_nr__) ({register char *__addr_temp__=(char *)(__dest_addr__);asm("bset.b %2,%0":"+dm"(*__addr_temp__):"0"(*__addr_temp__),"di"(__bit_nr__));})
#define peekIO_bit(port,bit) (!!(*((volatile unsigned char*)(long)(port))&(1<<(bit))))
#define IsEnabledAutoInt3() (peek_bit(0x600015,2))
#define EnableAutoInt3() (pokeIO_bset(0x600015,2))
#define DisableAutoInt3() (pokeIO_bclr(0x600015,2))
#endif

#define TRAP_4 0x90

#ifdef TEST_W_5
 #define INTERUPT AUTO_INT_5
#else
 #define INTERUPT AUTO_INT_3
#endif
int _ti89;  // produce code for TI89 only

// clock info structure:
unsigned short seconds		= 0; // counts seconds
unsigned short minutes		= 0; // counts minutes
unsigned short hours			= 0; // counts hours
unsigned short month			= 1; // counts days
unsigned short day				= 1; // counts months
unsigned short year				= 2; // counts years
unsigned short x = 132;		//x position to display clock
unsigned short y = 95;		//y position to display clock

short calcstate=0;	//a variable indicating the calc is:
										// 0: on
										// 1: off, ready to turn back on
										//>1: off

INT_HANDLER oldint3 = NULL;  // the old timer 3 interrupt(AutoInt3)
INT_HANDLER oldint6 = NULL;  // the old ON key interrupt (AutoInt6)
INT_HANDLER oldTrap4Hndlr = NULL;  // the old OFF trap (Trap4)

//=============================================================================
// outputs the time to the statusline
//=============================================================================
void PrintTime(void) {
	short font;
	char buffer[16];
	sprintf(buffer,"%02d:%02d:%02d", hours, minutes, seconds);
	font = FontSetSys(F_4x6);
	DrawStrXY(x, y, buffer, A_REPLACE);
	FontSetSys(font);
// The following is used to identify the clock from an external application
	(void)(*((unsigned char*)(long)(0x5500))=(42));							//verification header
	(void)(*((unsigned long*)(long)(0x5502))=((long)&seconds)); //and pointer to time
}

//=============================================================================
// our own interrupt handler hooked on interrupt 3 (updates clock variables)
//=============================================================================
const unsigned short numDays[]={31,28,31,30,31,30,31,31,30,31,30,31};
DEFINE_INT_HANDLER (clockHandler) {
	if(++seconds==60) {
		seconds = 0;
		if(++minutes==60) {
			minutes = 0;
			if(++hours==24) {
				hours = 0;
				if(day++==numDays[month-1]+(month==2&&!(year%4))) { //You'll have to reset the date every century
					day = 1;																					//since leap years are skipped every 100 years
					if(++month==13) {																	//2000 was another exception
						month = 1;
						year++;
					}
				}
			}
		}
	}
	//---------------------------------------------------------
	// execute the old interrupt handler at the end of our own
	// note to self: is this necessary for int 3?
	//---------------------------------------------------------
	if(!calcstate) { //again, only if calc is on
		PrintTime();  //Print time only if calc is on
#ifdef TEST_W_5
		ExecuteHandler(oldint3);
#endif
	} else {
		if(calcstate>1) calcstate--;		//counts down until calc can be turned on again, see below
//Currently, the link port will also turn on the calculator. Perhaps this should be disabled?
//more testing needed
#ifdef TEST_W_5
		pokeIO(0x600005,31); //Shut off CPU until ON key or Int 5
#else
		pokeIO(0x600005,0x0C); //Shut off CPU until ON key or Int 3
#endif
	}
}

DEFINE_INT_HANDLER (int6Handler) {
	if(calcstate>1) {     //If calculator hasn't been off long enough (see below)
		pokeIO(0x60001A,0);   //Clear ON Interrupt
#ifdef TEST_W_5           //and turn CPU off again
		pokeIO(0x600005,31); //until ON or Int 5
#else
		pokeIO(0x600005,0x0C); //until ON or Int 3
#endif
	} else {
		calcstate=0;	//calculator is on

		pokeIO(0x600015,0x1F);	//turn on all counters
#ifdef TEST_W_5							//turn on display
		pokeIO_bclr(0x60001D,4);	//assume HW1 for debugging
#else
		pokeIO_bset(0x70001D,1);	//HW2 for actual hook
#endif
		pokeIO(0x60001C,0x21);		//turn on row sync
		pokeIO_w(0x600018,0x0380);//turn on keyboard

		SetIntVec(AUTO_INT_6,oldint6);
		ExecuteHandler(oldint6);
	}
}
DEFINE_INT_HANDLER (trap4Handler) {
	calcstate=3;	//calcstate is used to debounce the on switch. when the
								//calculator gets turned off, the user must wait at least
								//calcstate-1 seconds* before turning it on again
								//without this, calculator would often just come on again
								//after user pressed off
								//* actually, occurances of AutoInt3
	pokeIO_w(0x600018,0x03FF);  //turn off keyboard
	pokeIO(0x60001C,0xFF);			//turn off row sync
#ifdef TEST_W_5								//turn off display
	pokeIO_bset(0x60001D,4);
#else
	pokeIO_bclr(0x70001D,1);
	pokeIO(0x600015,0x16);			//turn off all counters except 3
#endif

	oldint6 = GetIntVec(AUTO_INT_6);		//use our own ON-key handler
	SetIntVec(AUTO_INT_6,int6Handler);	//to turn calc back on later
	pokeIO(0x60001A,0xFF);		//Clear ON Interrupt
#ifdef TEST_W_5							//stop CPU
	pokeIO(0x600005,31);		//until ON or int5
#else
	pokeIO(0x600005,0x0C);	//until ON or int3
#endif
}

//=============================================================================
// the main routine of the TSR program
//
// NOTE: this routine is called twice !!
//
// One time to install our handler and a second time to remove it again
//=============================================================================
void _main(void) {
	if (oldint3) {
		//--------------------------------------------------
		// if TSR is already running (oldint3 != 0), stop it
		// by restoring the old interrupt vectors
		//--------------------------------------------------
#ifndef TEST_W_5
		DisableAutoInt3();
#else
		pokeIO(0x600017,0xCC);		//speed up timer 5 again (debug)
#endif
		SetIntVec(INTERUPT, oldint3);
		SetIntVec(TRAP_4,oldTrap4Hndlr);
		oldint3 = NULL; // make sure that oldint3 is zero afterwards
	} else {
		//-----------------------------------------------------------
		// the TSR is not running till now
		// backup the old interrupt handler address in global oldint5
		// and install our own handler
		//-----------------------------------------------------------
		oldint3 = GetIntVec(INTERUPT);			//setup the counting interrupt
		SetIntVec(INTERUPT, clockHandler);

		oldTrap4Hndlr = GetIntVec(TRAP_4);	//setup our own OFF handling routine
		SetIntVec(TRAP_4,trap4Handler);
#ifndef TEST_W_5
		EnableAutoInt3();
#else
		pokeIO(0x600017,1);		//slow down timer 5 as much as is easily possible
#endif
	}
}
