/******************************************************************************
*
* TI Clock Util, programmed by Johannes Zeppenfeld
*
*******************************************************************************
* For use with the ClockTi TSR
*******************************************************************************
* You may use this code however you like, but please give credit to those who
* wrote it.
* Other than that, this code is e-note-ware: if you use it, please send me an
* email telling me about your project and how you integrated the clock into it.
* I love to hear what others use programs I write for.
* Johannes Zeppenfeld (jzeppenfeld@excite.com)
******************************************************************************/
#define USE_TI89              // Produce .89z File
#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization
#include <tigcclib.h>         // Include All Header Files

struct Time {
	unsigned short seconds,minutes,hours;
	unsigned short month,day,year;
	unsigned short x,y;
};

//=============================================================================
//	finds and returns address of clock data structure
//=============================================================================
struct Time *ClockFind(void) {
	long i = 500000;	//500000 give a loop delay of just over 1 second
	poke(0x5500,0);
	while(peekIO(0x5500)!=42) { //wait until clock tsr has asserted its presence
		if(!i--) return NULL;     //or until time out
	}
	return *((void **)0x5502);
}

// Main Function
void _main(void) {
	HANDLE dialog;
	char buf[25];			//buffer used to store dialog values
	struct Time *t;
	if(!(t=ClockFind())) {
		ST_showHelp("Clock not installed.");
		return;
	}

//Initialize buffer to current time returned by ClockFind
//Probably not very efficient, but it works.
	sprintf(buf,"%02d",t->hours);
	sprintf(buf+3,"%02d",t->minutes);
	sprintf(buf+6,"%02d",t->seconds);
	sprintf(buf+9,"%02d",t->month);
	sprintf(buf+12,"%02d",t->day);
	sprintf(buf+15,"%02d",t->year);
	sprintf(buf+18,"%03d",t->x);
	sprintf(buf+22,"%03d",t->y);

//See TIGCC documentation for dialog information
	dialog = DialogNewSimple(125,60);
	if(!dialog)	return;
	DialogAddTitle (dialog, "Clock Edit", BT_OK, BT_CANCEL);	
	DialogAddRequest (dialog, 10, 16, "Time:", 0, 2, 3);
	DialogAddRequest (dialog, 53, 16, ":", 3, 2, 3);
	DialogAddRequest (dialog, 80, 16, ":", 6, 2, 3);
	DialogAddRequest (dialog, 11, 26, "Date:", 9, 2, 3);
	DialogAddRequest (dialog, 51, 26, "/", 12, 2, 3);
	DialogAddRequest (dialog, 78, 26, "/", 15, 2, 3);
	DialogAddRequest (dialog, 20, 36, "x:", 18, 3, 4);
	DialogAddRequest (dialog, 60, 36, "y:", 22, 3, 4);
	if (DialogDo (dialog, CENTER, CENTER, buf, NULL) == KEY_ENTER) {
		t->hours = atoi(buf);				//if user pressed OK, set clock
		t->minutes = atoi(buf+3);
		t->seconds = atoi(buf+6);
		t->month = atoi(buf+9);
		t->day = atoi(buf+12);
		t->year = atoi(buf+15);
		t->x = atoi(buf+18);
		t->y = atoi(buf+22);
	}
	HeapFree(dialog);	//free dialog to avoid memory leaks and return
}
